#!/usr/bin/perl -w

# this script just call SNPs, and will not do pedcheck
my $min_n_homo = 3;
my $min_n_minor = 2;
my $min_p_minor = 0.2;
# at least $n_min members must be involved in he locus
my $n_min = 2;

my $sam_file = shift;
# file for naming samples in the format of:
# Fish_name	print_name
open(SAM, $sam_file) or die "Unable to open $sam_file\n";
my @sam_list = ();
my @names = ();
while (<SAM>) {
    chomp;
    my @tmp = split /\s+/;
    push @sam_list, $tmp[0];
    push @names, $tmp[1];
}
close(SAM);

# re-hash output of sequence member file
my $dat_file = shift;
open(MEM, $dat_file) or die "Unable to open file: $dat_file.\n";
my %reads = ();
my %samples = ();
while (<MEM>) {
    chomp;
    my @tmp = split /\t/;
    $reads{$tmp[0]} = $tmp[1];
    $samples{$tmp[0]} = $tmp[2];
}
close(MEM);

# the input file is the one output by find_bi_allelic.pl
my $in = shift;
my $out = shift;
my $fna = shift;
my $max_n_snp = shift;   # maxmum number of SNP sites allowed
open(IN, $in) or die "Unable to open file: $in.\n";
open(OUT, ">$out") or die "Unable to open file: $out.\n";
open(FNA, ">$fna") or die "Unable to open file: $fna.\n";
# output header
my @headers = ("Locus", "SeqID", "major", "minor", "n_SNP", "SNPs", "fpass", "hetz", "ntype", @names);
my $first_row = join "\t", @headers;
print OUT $first_row, "\n";

my $row=0;
my $nprnt = 0;
my @counts = ();
<IN>;
while (<IN>) {
    chomp;
    my @clns = split /\t/;
    my $qry = $clns[0];
    my $ref = $clns[1];
    my %diff = ();
    ($diff{$qry}, $diff{$ref}) = ($clns[2], $clns[3]);
    my %seq = ();
    ($seq{$qry}, $seq{$ref}) = ($clns[4], $clns[5]);
    my %poly = ();
    ($poly{$qry}, $poly{$ref}) = ("", "");
    my @sites = split / /, $clns[2];
    foreach my $site (@sites) {
        $site =~ s/\d|\W//g;
        my @lett = split //, $site;
        $poly{$qry} .= $lett[1];
        $poly{$ref} .= $lett[0];
    }
    my $major = $qry;
    my $minor = $ref;
    if ($reads{$qry} < $reads{$ref}) {
        $major = $ref;
        $minor = $qry;
    }
    my $ratio = $reads{$major} . '/' . $reads{$minor};
    my $cmajor = $reads{$major};
    my $cminor = $reads{$minor};
    # check how many SNPs
    my $n_snp = scalar(@sites);
    if ($n_snp <= $max_n_snp) {
        # check number of samples
        my $n_samp = 0;
        my %gtype = ();    # 1 , in major; 2, in minor; 3, in both
        my %rmajor = ();
        my %rminor = ();
        my @sams = split /\//, $samples{$major};
        my %seen_sam = ();
        foreach my $sm (@sams) {
            my @stmp = split /\;/, $sm;
            my $spn = $stmp[0];
            $spn =~ s/^>//;
            $gtype{$spn} = 1;
            $rmajor{$spn} += $stmp[2];   # here += takes the case same sample appears in an allele more than 1 times in the .dat file 
            unless (defined($seen_sam{$spn})) {
                $n_samp++;
                $seen_sam{$spn} = 1;
            }
        }
        @sams = split /\//, $samples{$minor};
        foreach my $sm (@sams) {
            my @stmp = split /\;/, $sm;
            my $spn = $stmp[0];
            $spn =~ s/^>//;
            if (defined($gtype{$spn}) and $gtype{$spn} == 1) {    # make sure gtype = 3 is for hereozygous
                $gtype{$spn} = 3;
            } else {
                $gtype{$spn} = 2;
                unless (defined($seen_sam{$spn})) {
                    $n_samp++;
                    $seen_sam{$spn} = 1;
                }
            }
            $rminor{$spn} += $stmp[2];    # += is for the same reason as above
        }
        if ($n_samp >= $n_min) {
            # check genotype
            my %alle1 = ();
            my %alle2 = ();
            my %rpass = ();    # =1, passed read count; =0 not passed read count
            foreach my $sample (@sam_list) {
                $rpass{$sample} = 0;
                if (defined($gtype{$sample})) {
                    if ($gtype{$sample} == 1) {
                        $alle1{$sample} = $poly{$major};
                        $alle2{$sample} = $poly{$major};
                        if ($rmajor{$sample} >= $min_n_homo) {
                            $rpass{$sample} = 1;
                        }
                    } elsif ($gtype{$sample} == 2) {
                        $alle1{$sample} = $poly{$minor};
                        $alle2{$sample} = $poly{$minor};
                        if ($rminor{$sample} >= $min_n_homo) {
                            $rpass{$sample} = 1;
                        }
                    } elsif ($gtype{$sample} == 3) {
                        $alle1{$sample} = $poly{$major};
                        $alle2{$sample} = $poly{$minor};
                        my $nrd;
                        my $prd;
                        unless (defined($rmajor{$sample}) and defined($rminor{$sample})) {
                            print $sample, "\t", $gtype{$sample}, "\t", $major, "\t", $minor, "\n";
                            exit;
                        }
                        if ($rmajor{$sample} >= $rminor{$sample}) {
                            $nrd = $rminor{$sample};
                            $prd = $rminor{$sample} / ($rmajor{$sample} + $rminor{$sample});
                        } else {
                            $nrd = $rmajor{$sample};
                            $prd = $rmajor{$sample} / ($rmajor{$sample} + $rminor{$sample});
                        }
                        if ($prd >= $min_p_minor and $nrd >= $min_n_minor) {
                            $rpass{$sample} = 1;
                        }
                    }
                }
            }
            # $fpass ---- total passed fish
            # $ihetz ---- number of fish with heterozygosity
            my $fpass = 0;
            my $ihetz = 0;
            my %type = ();   # check how many unique types of gnotype
            foreach my $fish (@sam_list) {
              if ($rpass{$fish} == 1) {
                  ++$fpass;
                  if ($alle1{$fish} ne $alle2{$fish}) {
                      ++$ihetz; 
                  }
                  unless(defined($type{$gtype{$fish}})) {
                      $type{$gtype{$fish}} = 1;
                  }
              }
            }
            my $ntype = scalar(keys %type);   # 1 - homozygous; > 1 - SNP 
            my $ihomo = $fpass - $ihetz;

            if ($fpass >= $n_min and $ntype > 1) {
               foreach my $fish (@sam_list) {
                   if ($rpass{$fish} == 1) {
                       ++$counts{$fish};
                   }
               }
               ++$nprnt;
               print OUT "$nprnt\t$major\t$cmajor\t$cminor\t$n_snp\t",
                      $diff{$major}, "\t$fpass\t$ihetz\t$ntype";
               for my $sample (@sam_list) {
                   if ($rpass{$sample} == 1) {
                       print OUT "\t", $alle1{$sample}, "\/", $alle2{$sample};
                   } else {
                       print OUT "\t0";
                   }
               }
               print OUT "\n";
               print FNA ">l$nprnt\;$major\;", $diff{$major}, "\n";
               print FNA $seq{$major}, "\n";
            }
        }
    }
    ++$row;
}
close(IN);
close(OUT);
close(FNA);

print "Total input loci: $row\n";
print "    Printed rows: $nprnt\n";

foreach my $fish (keys %counts) {
    print $fish, "\t", $counts{$fish}, "\n";
}
exit;
