#!/usr/bin/perl -w
# we use this script to select the sequences that exactly match the barcodes 
# plus the restriction enzyme recognition sites for the samples and have the 
# probability larger than 80%
# the output doesn't include the barcodes

# some parameters to adjust 
# for SbfI: 
# my $rcode = 'TGCAGG';
# for PstI
# my $rcode = 'TGCAG';
my $rcode = shift;
my $rcut = shift;
print "Restrict enzyme code: $rcode\n";

my $percent_filter = 80;
#The value should be either 33 or 64 depending on the base-calling pipeline
my $phred = 33;  

# barcode file
my $in = shift;
open (IN, $in);
my %barcode = ();
my $che_start = 12;
my $out_start = 6;
while (<IN>) {
    chomp;
    my @tmp = split /\t/;
    my $s12 = $tmp[1] . $rcode;
    $che_start = length $s12;
    $out_start = length $tmp[1];
    $barcode{$s12} = $tmp[0];
}
close(IN);
print "Offset for sequence checking: $che_start\n";
print "  Offset for sequence output: $out_start\n";
print "          Base pairs trimmed: $rcut\n\n";

# input fastq file for raw sequences
$in = shift;
print "Input file: $in\n";
open (IN, $in) or die "Unable to open input file $in\n";
# output file for the selected sequences
my $out = shift;
open(OUT, ">$out") or die "Couldn't open file $out\n";

my %count = ();
my $total = 0;
my $nwell = 0;
my %stat = ();
my %p = ();
my $good_reads = 0;

while (my $tag = <IN>) {
    my $seq = <IN>;
    my $hhh = <IN>;
    my $qqq = <IN>;
    chomp($seq);
    chomp($hhh);
    chomp($qqq);
    ++$total;
    my $bar = substr($seq, 0, $che_start);
    if (defined($barcode{$bar})) {
        $count{$bar}++;
        ++$nwell;
        my $len = length $seq;
        $len = $len - $rcut;
        $seq = substr($seq, 0, $len);
        unless ($seq =~ m/N/) {
            my $qual_chk = substr($qqq,$che_start,$len-$che_start);
            my @ASCII = unpack("C*", $qual_chk);
            my $prob = 100;
            foreach my $value (@ASCII) {
                $value = $value - $phred;
                $prob = $prob * (1-(10**(-$value/10)));
            }
            if ($prob >= $percent_filter) {
               ++$stat{$bar};
               ++$good_reads;
               $p{$bar} += $prob;
               $hhh .= $barcode{$bar} . " Prob=" . $prob;
               print OUT $tag;
               print OUT substr($seq, $out_start, $len-$out_start), "\n";
               print OUT $hhh, "\n";
               print OUT substr($qqq, $out_start, $len-$out_start), "\n";
            }
        }
    }
}
close(IN);
close(OUT);

print "                      Total sequences: $total\n";
print " # of sequencesi passed barcode check: $nwell\n";
print "# of sequencesi passed quality filter: $good_reads\n";
print "\nNumber of sequences by samples:\n";
print "Sample\tBarcode\tCode_match\tQFilter\tP_avg\n";

foreach my $bar (sort keys %count) {
    my $pavg;
    if (defined($stat{$bar})) {
        $pavg = $p{$bar} / $stat{$bar};
    } else {
        $pavg = 0;
    }
    print $barcode{$bar}, "\t", $bar, "\t", $count{$bar}, "\t", $stat{$bar}, "\t", $pavg, "\n";
}
exit;
